#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import numpy as np
from jhora import const, utils

planet_list = ['sun','moon','mars','mercury','jupiter','venus','saturn','lagnam']
raasi_list=['Mesham','Rishabam','Mithunam','Katakam','Simmam','Kanni','Thulaam','Vrichigam','Dhanusu','Makaram','Kumbam','Meenam']
raasi_index = lambda planet,planet_positions_in_chart: [i for i,raasi in enumerate(planet_positions_in_chart) if planet !=const._ascendant_symbol and planet.lower() in raasi.lower() ][0]
def get_ashtaka_varga(house_to_planet_list):
    """
        get binna, samudhaya and prastara varga from the given horoscope chart
        @param house_to_planet_list: 1-D array [0..11] with planets in each raasi
            Example: ['','','','','2','7','1/5','0','3/4','L','','6/8']
        @return: 
            binna_ashtaka_varga - 2-D List [0..7][0..7] 0=Sun..7=Lagnam
            samudhaya ashtaka varga - 1D List [0..11] 0=Aries 11=Pisces
            prastara ashtaka varga - 3D List [0..7][0..7][0..11]
    """
    #print('get_ashtaka_varga','house_to_planet_list',house_to_planet_list)
    p_to_h = utils.get_planet_to_house_dict_from_chart(house_to_planet_list)
    #print('get_ashtaka_varga','p_to_h',p_to_h)
    raasi_ashtaka = [[0 for r in range(12)] for p in range(8)]
    prastara_ashtaka_varga  = [[[0 for r in range(12)] for p1 in range(10)] for p2 in range(8)]
    for key in const.ashtaka_varga_dict.keys():
        p = int(key)
        #planet = planet_list[p]
        planet_raasi_list = const.ashtaka_varga_dict[key]
        for op,other_planet in enumerate(planet_raasi_list):
            pr = p_to_h[op]
            if op == 7: #Lagnam
                pr = p_to_h[const._ascendant_symbol]
            for raasi in other_planet:
                r = (raasi-1+pr) % 12
                raasi_ashtaka[p][r] +=1
                prastara_ashtaka_varga[p][op][r] = 1
                prastara_ashtaka_varga[p][-1][r] += 1
    binna_ashtaka_varga = raasi_ashtaka[0:8][:]
    prastara_ashtaka_varga = prastara_ashtaka_varga[0:8][0:9][:]
    samudhaya_ashtaka_varga = np.asarray(binna_ashtaka_varga[:-1]).sum(axis=0).tolist() # [0:-1] to exlcude Lagnam
    return binna_ashtaka_varga, samudhaya_ashtaka_varga,prastara_ashtaka_varga
def _trikona_sodhana(binna_ashtaka_varga):
    bav = binna_ashtaka_varga[:]
    for p in range(7):
        for r in range(4):
            if bav[p][r]==0 or bav[p][r+4]==0 or bav[p][r+8]==0:
                #print('Rule 1:If atleast one rasi has zero, no reduction is necessary.',p,r,bav[p][r],bav[p][r+4],bav[p][r+8])
                continue
            elif bav[p][r]==bav[p][r+4] and bav[p][r+4]==bav[p][r+8]:
                #print('Rule (2) If the three rasis have the same value, make them all zero.',p,r,bav[p][r],bav[p][r+4],bav[p][r+8])
                bav[p][r]=0
                bav[p][r+4]=0
                bav[p][r+8]=0
            else:
                #print('Rule (3) Take the lowest value out of the three. Subtract it from all the values.',p,r,bav[p][r],bav[p][r+4],bav[p][r+8])
                min_value = min([bav[p][r],bav[p][r+4],bav[p][r+8]])
                #print('before',p,r,min_value,bav[p][r],bav[p][r+4],bav[p][r+8])
                bav[p][r] -= min_value
                bav[p][r+4] -= min_value
                bav[p][r+8] -= min_value
                #print('after',p,r,min_value,bav[p][r],bav[p][r+4],bav[p][r+8])
    return bav
def _ekadhipatya_sodhana(binna_ashtaka_varga_after_trikona,chart_1d):
    bav = binna_ashtaka_varga_after_trikona[:]
    rasi_owners=[4,3,(0,7),(2,5),(8,11),(1,6),(9,10)]
    for p in range(2,7):
        r1,r2 = rasi_owners[p]
        r1_occupied = not (chart_1d[r1].strip() == '')
        r2_occupied = not (chart_1d[r2].strip() == '')
        # Rule 1 (either bav is 0. Rule 2: either rasi is occupied by a planet
        if (bav[p][r1]==0 or bav[p][r2]==0) or (r1_occupied and r2_occupied) :
            continue
        elif (not r1_occupied) and (not r2_occupied ): # Rule 4 - both rasi are empty
            # Rule 4(b) (b) If they have different values, replace the higher value with the lower value.
            if bav[p][r1] != bav[p][r2]: 
                min_value = min([bav[p][r1],bav[p][r2]])
                bav[p][r1] = min_value
                bav[p][r2] = min_value
            # Rule 4(a) If both the rasis have the same value, replace both with zero.
            else:
                bav[p][r1] = 0
                bav[p][r2] = 0
        else:
            # Rule (3) If one rasi is occupied by a planet (or planets) and the other is empty
            if (r1_occupied): #r2 is empty
                # Rule 3(a) If the empty rasi has a lower value, replace the value with a zero.
                if (bav[p][r2] < bav[p][r1]):
                    bav[p][r2] = 0
                # Rule 3(b) If the empty rasi has a higher value, replace the value with the value in the other rasi.
                else:
                    bav[p][r2] = bav[p][r1]
            else: #r1 is empty
                # Rule 3(a) If the empty rasi has a lower value, replace the value with a zero. 
                if (bav[p][r1] < bav[p][r2]):
                    bav[p][r1] = 0
                # Rule 3(b) If the empty rasi has a higher value, replace the value with the value in the other rasi.
                else:
                    bav[p][r1] = bav[p][r2]
    return bav
def _get_planet_positions(chart_1d):
    planet_houses = [-1 for p in range(7)]
    for p,planet in enumerate(planet_list[0:-1]): # Excluding Lagnam
        for house,rasi in enumerate(chart_1d):
            if planet.lower() in rasi.lower():
                planet_houses[p] = house
                break
    return planet_houses
def _sodhya_pindas(binna_ashtaka_varga_after_ekadhipatya,chart_1d):
    rasimana_multipliers = [7,10,8,4,10,6,7,8,9,5,11,12]
    grahamana_multipliers = [5,5,8,5,10,7,5]
    bav = binna_ashtaka_varga_after_ekadhipatya[:]
    raasi_pindas = [0 for p in range(7)]
    graha_pindas = [0 for p in range(7)]
    sodhya_pindas = [0 for p in range(7)]
    for p in range(7):
        raasi_pindas[p] = sum(np.multiply(bav[p][:],rasimana_multipliers))
    #planet_houses = _get_planet_positions(chart_1d)
    planet_houses = list(utils.get_planet_to_house_dict_from_chart(chart_1d).values())[:7] # Exclude Rahu, Ketu and Lagnam
    for p in range(7):
        graha_pindas[p] = sum([ grahamana_multipliers[i]*bav[p][pr] for i,pr in enumerate(planet_houses)])
        sodhya_pindas[p] = raasi_pindas[p]+graha_pindas[p]
    return raasi_pindas,graha_pindas,sodhya_pindas
def sodhaya_pindas(binna_ashtaka_varga,house_to_planet_chart):
    """
        Get sodhaya pindas from binna ashtaka varga
        @param param:binna_ashtaka_varga - 2-D List [0..7][0..7] 0=Sun..7=Lagnam - of BAV values
        NOTE: To pass binn ashtaka varga as parameter - you need to get it from get_ashtaka_varga function
        @return: raasi_pindas,graha_pindas,sodhya_pindas
                raasi_pindas : raasi pindas of planets 0=Sun to 6=Saturn [0..6]
                graha_pindas : graha pindas of planets 0=Sun to 6=Saturn [0..6]
                sidhaya_pindas : sodhaya pindas of planets 0=Sun to 6=Saturn [0..6]
    """
    #print('bav before trikona',binna_ashtaka_varga)
    binna_ashtaka_varga_after_trikona = _trikona_sodhana(binna_ashtaka_varga)
    #print('bav after trikona',binna_ashtaka_varga_after_trikona)
    binna_ashtaka_varga_after_ekadhipatya = _ekadhipatya_sodhana(binna_ashtaka_varga_after_trikona,house_to_planet_chart)
    """ binna_ashtaka_varga_after_ekadhipatya is called Sodhita Ashtakavarga"""
    #print('Sodhita Ashtakavarga\n',binna_ashtaka_varga_after_ekadhipatya)
    raasi_pindas,graha_pindas,sodhya_pindas = _sodhya_pindas(binna_ashtaka_varga_after_ekadhipatya,house_to_planet_chart)
    return raasi_pindas,graha_pindas,sodhya_pindas
if __name__ == "__main__":
    from jhora.tests.pvr_tests import test_example
    # Chart 7 from the book
    chapter = 'Chaper 12.3 ashtaka_varga_tests'
    exercise = 'Exercise 22/Chart 7:'
    chart_7 = ['6/1/7','','','','','','8/4','L','3/2','0','5','']
    bav, sav,pav = get_ashtaka_varga(chart_7)
    bav_e = [[4,2,3,4,6,5,5,3,2,6,6,2],
             [6,3,5,3,5,5,6,3,3,4,4,2],
             [3,2,3,4,2,5,4,3,3,4,3,3],
             [4,6,4,3,4,7,4,5,6,3,5,3],
             [4,4,3,5,6,5,6,4,6,4,3,6],
             [3,5,5,4,6,2,3,6,5,2,7,4],
             [3,2,2,3,5,6,3,4,1,3,6,1]]
    test_example(chapter+exercise+' BAV',bav_e,bav[:-1])#,assert_result=True)
    sav_e = [27,24,25,26,34,35,31,28,26,26,34,21]
    test_example(chapter+exercise+' SAV',sav_e,sav)#,assert_result=True)
    sp_e_book = [[152,85,52,95,68,154,162],[81,55,43,33,56,54,63],[233,140,95,128,124,208,225]]
    sp_e = ([155, 92, 55, 99, 93, 154, 166], [81, 55, 43, 33, 56, 54, 63], [236, 147, 98, 132, 149, 208, 229])
    sp = sodhaya_pindas(bav, chart_7)
    test_example(chapter+exercise+' Sodhaya Pindas',sp_e,sp)
    print(chapter+exercise+' Sodhaya Pindas:\n NOTE: Not clear why this case SP failed to match the book\n'+
          ' Examples 40,41 & 42 based on Chart 12 are matching BAV, SAV and SP.\n So the calculations in this code is thus verified\n'+
          'Expected Values from Book:',sp_e_book)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
    Sandhya is another Ayurdasa system. Concept: Sandhya is the Dvadashāńśa Ayurdaya of the Param Ayurdaya. 
    In this dasa system, the parama-ayush is spread among the 12 Rāśis, making the dasa span of each Rāśi as 1/12th of the Paramaayush. 
    For humans the Paramayush have been agreed by savants as 120 years. Hence the span of each Sandhya Dasa is 10 years. 
    
    Also includes Panchaka Dasa Variation - wherein 10 years are divided into 3 compartments: 
    1 rasi - 61/30, 3 rasis-61/60 and 8 rasis - 61/90  - each fraction of 10 years 
"""
from jhora import utils, const
from jhora.panchanga import drik
from jhora.horoscope.chart import charts

_sandhya_duration = [10 for _ in range(12)]
_panchaka_duration = [60/31,30/31,30/31,30/31,20/31,20/31,20/31,20/31,20/31,20/31,20/31,20/31]

def get_dhasa_antardhasa(dob,tob,place,divisional_chart_factor=1,years=1,months=1,sixty_hours=1,
                         include_antardhasa=False,use_panchaka_variation=False):
    jd_at_dob = utils.julian_day_number(dob, tob)
    pp = charts.divisional_chart(jd_at_dob, place, divisional_chart_factor=divisional_chart_factor, years=years, months=months,sixty_hours=sixty_hours)
    _dhasa_seed = pp[0][1][0]
    _dhasa_progression = [((_dhasa_seed+h)%12,_durn) for h,_durn in enumerate(_sandhya_duration)]
    start_jd = jd_at_dob
    dhasa_info = []
    for dhasa_lord, dhasa_duration in _dhasa_progression:
        if include_antardhasa:
            bhukthi_duration = dhasa_duration/12.0
            for bhukthi_lord in [(dhasa_lord+h)%12 for h in range(12)]:
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,round(bhukthi_duration,2)))
                start_jd += bhukthi_duration * const.sidereal_year
        elif use_panchaka_variation:
            for b,bhukthi_lord in enumerate([(dhasa_lord+h)%12 for h in range(12)]):
                bhukthi_duration = _panchaka_duration[b]
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,round(bhukthi_duration,2)))
                start_jd += bhukthi_duration * const.sidereal_year
        else:
            y,m,d,h = utils.jd_to_gregorian(start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasa_info.append((dhasa_lord,dhasa_start,round(dhasa_duration,2)))
            start_jd += dhasa_duration * const.sidereal_year
    return dhasa_info
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests.sandhya_test()        


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora import const, utils
from jhora.horoscope.chart import house
"""
    Saham calculation
    saham has a formula that looks like A – B + C. What this means is that we take the
    longitudes of A, B and C and find (A – B + C). This is equivalent to finding how far
    A is from B and then taking the same distance from C. However, if C is not between
    B and A (i.e. we start from B and go zodiacally till we meet A and we do not find C
    on the way), then we add 30º to the value evaluated above.    
"""
saham_longitude = lambda pp,p:pp[p][1][0]*30+pp[p][1][1]
lagna_longitude = lambda pp: saham_longitude(pp,0)
sun_longitude = lambda pp: saham_longitude(pp,1)
moon_longitude = lambda pp: saham_longitude(pp,2)
mars_longitude = lambda pp: saham_longitude(pp,3)
mercury_longitude = lambda pp: saham_longitude(pp,4)
jupiter_longitude = lambda pp: saham_longitude(pp,5)
venus_longitude = lambda pp: saham_longitude(pp,6)
saturn_longitude = lambda pp: saham_longitude(pp,7)

# Saham Meaning Formula
def punya_saham(planet_positions,night_time_birth=False):
# 1 Punya Fortune/good deeds Moon – Sun + Lagna
    moon_long = moon_longitude(planet_positions)
    sun_long = sun_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    punya_sagam_long = moon_long - sun_long + lagna_long
    if not _is_C_between_B_to_A(moon_long,sun_long,lagna_long):
        punya_sagam_long += 30
    if night_time_birth:
        punya_sagam_long = sun_long - moon_long + lagna_long
        if not _is_C_between_B_to_A(sun_long,moon_long,lagna_long):
            punya_sagam_long += 30
    punya_sagam_long %= 360
    return punya_sagam_long
    
def vidya_saham(planet_positions,night_time_birth=False):
# 2 Vidya Education Sun – Moon + Lagna
    sun_long = sun_longitude(planet_positions)
    moon_long = moon_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    vidya_sagam_long = sun_long - moon_long + lagna_long
    if not _is_C_between_B_to_A(sun_long,moon_long,lagna_long):
        vidya_sagam_long += 30
    if night_time_birth:
        vidya_sagam_long = moon_long - sun_long + lagna_long
        if not _is_C_between_B_to_A(moon_long,sun_long,lagna_long):
            vidya_sagam_long += 30
    vidya_sagam_long %= 360
    return vidya_sagam_long
def yasas_saham(planet_positions,night_time_birth=False):
# 3 Yasas Fame Jupiter – PunyaSaham + Lagna
    jupiter_long = jupiter_longitude(planet_positions)
    punya_long = punya_saham(planet_positions, night_time_birth)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    yasas_sagam_long = jupiter_long - punya_long + lagna_long
    if not _is_C_between_B_to_A(jupiter_long,punya_long,lagna_long):
        yasas_sagam_long += 30
    if night_time_birth:
        yasas_sagam_long = punya_long - jupiter_long + lagna_long
        if not _is_C_between_B_to_A(punya_long,jupiter_long,lagna_long):
            yasas_sagam_long += 30
    yasas_sagam_long %= 360
    return yasas_sagam_long
def mitra_saham(planet_positions,night_time_birth=False):
# 4 Mitra Friend Jupiter – PunyaSaham + Venus
    jupiter_long = jupiter_longitude(planet_positions)
    punya_long = punya_saham(planet_positions, night_time_birth)
    venus_long = venus_longitude(planet_positions)
    " A - B + C "
    mitra_sagam_long = jupiter_long - punya_long + venus_long
    if not _is_C_between_B_to_A(jupiter_long,punya_long,venus_long):
        mitra_sagam_long += 30
    if night_time_birth:
        mitra_sagam_long = punya_long - jupiter_long + venus_long
        if not _is_C_between_B_to_A(punya_long,jupiter_long,venus_long):
            mitra_sagam_long += 30
    mitra_sagam_long %= 360
    return mitra_sagam_long
def mahatmaya_saham(planet_positions,night_time_birth=False):
# 5 Mahatmya Greatness PunyaSaham – Mars + Lagna
    punya_long = punya_saham(planet_positions, night_time_birth)
    mars_long = mars_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    mahatmaya_sagam_long = punya_long - mars_long + lagna_long
    if not _is_C_between_B_to_A(punya_long,mars_long,lagna_long):
        mahatmaya_sagam_long += 30
    if night_time_birth:
        mahatmaya_sagam_long = mars_long - punya_long + lagna_long
        if not _is_C_between_B_to_A(mars_long,punya_long,lagna_long):
            mahatmaya_sagam_long += 30
    mahatmaya_sagam_long %= 360
    return mahatmaya_sagam_long
def asha_saham(planet_positions,night_time_birth=False):
# 6 Asha Desires Saturn – Mars + Lagna
    saturn_long = saturn_longitude(planet_positions)
    mars_long = mars_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    asha_sagam_long = saturn_long - mars_long + lagna_long
    if not _is_C_between_B_to_A(saturn_long,mars_long,lagna_long):
        asha_sagam_long += 30
    if night_time_birth:
        asha_sagam_long = mars_long - saturn_long + lagna_long
        if not _is_C_between_B_to_A(mars_long,saturn_long,lagna_long):
            asha_sagam_long += 30
    asha_sagam_long %= 360
    return asha_sagam_long
def samartha_saham(planet_positions,night_time_birth=False):
# 7 Samartha Enterprise/ability    Mars – Lagna Lord + Lagna (Jupiter – Mars + Lagna, if Mars owns lagna)
    h_to_p = utils.get_house_planet_list_from_planet_positions(planet_positions)
    mars_long = mars_longitude(planet_positions)
    lagna_house = planet_positions[0][1][0]
    lagna_lord = house.house_owner_from_planet_positions(planet_positions,lagna_house)
    if lagna_lord == 2:
        #print('Lagna Lord is Mars. So Jupiter is used as Lagna Lord in saham equation')
        lagna_lord = 4 # Jupiter
        " This is done to swap ABC equation "
        night_time_birth = not night_time_birth
    lagna_lord_long = saham_longitude(planet_positions,lagna_lord+1)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    samartha_sagam_long = mars_long - lagna_lord_long + lagna_long
    if not _is_C_between_B_to_A(mars_long,lagna_lord_long,lagna_long):
        samartha_sagam_long += 30
    if night_time_birth:
        samartha_sagam_long = lagna_lord_long - mars_long + lagna_long
        if not _is_C_between_B_to_A(lagna_lord_long,mars_long,lagna_long):
            samartha_sagam_long += 30
    samartha_sagam_long %= 360
    return samartha_sagam_long
def bhratri_saham(planet_positions):
# 8 Bhratri Brothers Jupiter – Saturn + Lagna (same for day & night)
    jupiter_long = jupiter_longitude(planet_positions)
    saturn_long = saturn_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    bhratri_sagam_long = jupiter_long - saturn_long + lagna_long
    if not _is_C_between_B_to_A(jupiter_long,saturn_long,lagna_long):
        bhratri_sagam_long += 30
    bhratri_sagam_long %= 360
    return bhratri_sagam_long
def gaurava_saham(planet_positions,night_time_birth=False):
    """
        TODO: JHora book says jupiter-moon+sun
        Internet says: sun-moon+jupiter - which is right. We follow JHora book - but does not match with JHora s/w.
    """
# 9 Gaurava Respect/regard Jupiter – Moon + Sun
    jupiter_long = jupiter_longitude(planet_positions)
    moon_long = moon_longitude(planet_positions)
    sun_long = sun_longitude(planet_positions)
    " A - B + C "
    gaurava_sagam_long = jupiter_long - moon_long + sun_long
    if not _is_C_between_B_to_A(jupiter_long,moon_long,sun_long):
        gaurava_sagam_long += 30
    if night_time_birth:
        gaurava_sagam_long = moon_long - jupiter_long + sun_long
        if not _is_C_between_B_to_A(moon_long,jupiter_long,sun_long):
            gaurava_sagam_long += 30
    gaurava_sagam_long %= 360
    return gaurava_sagam_long
def pithri_saham(planet_positions,night_time_birth=False):
    # 10 Pitri Father Saturn – Sun + Lagna
    saturn_long = saturn_longitude(planet_positions)
    sun_long = sun_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    pithri_sagam_long = saturn_long - sun_long + lagna_long
    if not _is_C_between_B_to_A(saturn_long,sun_long,lagna_long):
        pithri_sagam_long += 30
    if night_time_birth:
        pithri_sagam_long = sun_long - saturn_long + lagna_long
        if not _is_C_between_B_to_A(sun_long,saturn_long,lagna_long):
            pithri_sagam_long += 30
    pithri_sagam_long %= 360
    return pithri_sagam_long
def rajya_saham(planet_positions,night_time_birth=False):
# 11 Rajya Kingdom Saturn – Sun + Lagna
    return pithri_saham(planet_positions,night_time_birth)

def maathri_saham(planet_positions,night_time_birth=False):
# 12 Matri Mother Moon – Venus + Lagna
    moon_long = moon_longitude(planet_positions)
    venus_long = venus_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    maathri_sagam_long = moon_long - venus_long + lagna_long
    if not _is_C_between_B_to_A(moon_long,venus_long,lagna_long):
        maathri_sagam_long += 30
    if night_time_birth:
        maathri_sagam_long = venus_long - moon_long + lagna_long
        if not _is_C_between_B_to_A(venus_long,moon_long,lagna_long):
            maathri_sagam_long += 30
    maathri_sagam_long %= 360
    return maathri_sagam_long
def puthra_saham(planet_positions,night_time_birth=False):
# 13 Putra Children Jupiter – Moon + Lagna
    jupiter_long = jupiter_longitude(planet_positions)
    moon_long = moon_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    puthra_sagam_long = jupiter_long - moon_long + lagna_long
    if not _is_C_between_B_to_A(jupiter_long,moon_long,lagna_long):
        puthra_sagam_long += 30
    if night_time_birth:
        puthra_sagam_long = moon_long - jupiter_long + lagna_long
        if not _is_C_between_B_to_A(moon_long,jupiter_long,lagna_long):
            puthra_sagam_long += 30
    puthra_sagam_long %= 360
    return puthra_sagam_long
def jeeva_saham(planet_positions,night_time_birth=False):
# 14 Jeeva Life Saturn – Jupiter + Lagna
    saturn_long = saturn_longitude(planet_positions)
    jupiter_long = jupiter_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    jeeva_sagam_long = saturn_long - jupiter_long + lagna_long
    if not _is_C_between_B_to_A(saturn_long,jupiter_long,lagna_long):
        jeeva_sagam_long += 30
    if night_time_birth:
        jeeva_sagam_long = jupiter_long - saturn_long + lagna_long
        if not _is_C_between_B_to_A(jupiter_long,saturn_long,lagna_long):
            jeeva_sagam_long += 30
    jeeva_sagam_long %= 360
    return jeeva_sagam_long
def karma_saham(planet_positions,night_time_birth=False):
# 15 Karma Action (work) Mars – Mercury + Lagna
    mars_long = mars_longitude(planet_positions)
    mercury_long = mercury_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    karma_sagam_long = mars_long - mercury_long + lagna_long
    if not _is_C_between_B_to_A(mars_long,mercury_long,lagna_long):
        karma_sagam_long += 30
    if night_time_birth:
        karma_sagam_long = mercury_long - mars_long + lagna_long
        if not _is_C_between_B_to_A(mercury_long,mars_long,lagna_long):
            karma_sagam_long += 30
    karma_sagam_long %= 360
    return karma_sagam_long
def roga_saham(planet_positions,night_time_birth=False):
# 16 Roga Disease Lagna – Moon + Lagna (Same for night/day)
    lagna_long = lagna_longitude(planet_positions)
    moon_long = moon_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    roga_sagam_long = lagna_long - moon_long + lagna_long
    roga_sagam_long %= 360
    return roga_sagam_long
def roga_sagam_1(planet_positions,night_time_birth=False):
# 16 Roga Disease - Another Version -  Saturn – Moon + Lagna
    saturn_long = saturn_longitude(planet_positions)
    moon_long = moon_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    roga_sagam_long = saturn_long - moon_long + lagna_long
    if not _is_C_between_B_to_A(saturn_long,moon_long,lagna_long):
        roga_sagam_long += 30
    if night_time_birth:
        roga_sagam_long = moon_long - saturn_long + lagna_long
        if not _is_C_between_B_to_A(moon_long,saturn_long,lagna_long):
            roga_sagam_long += 30
    roga_sagam_long %= 360
    return roga_sagam_long
def kali_saham(planet_positions,night_time_birth=False):
# 17 Kali Great misfortune Jupiter – Mars + Lagna
    jupiter_long = jupiter_longitude(planet_positions)
    mars_long = mars_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    kali_sagam_long = jupiter_long - mars_long + lagna_long
    if not _is_C_between_B_to_A(jupiter_long,mars_long,lagna_long):
        kali_sagam_long += 30
    if night_time_birth:
        kali_sagam_long = mars_long - jupiter_long + lagna_long
        if not _is_C_between_B_to_A(mars_long,jupiter_long,lagna_long):
            kali_sagam_long += 30
    kali_sagam_long %= 360
    return kali_sagam_long
def sastra_saham(planet_positions,night_time_birth=False):
# 18 Sastra Sciences Jupiter – Saturn + Mercury
    jupiter_long = jupiter_longitude(planet_positions)
    saturn_long = saturn_longitude(planet_positions)
    mercury_long = mercury_longitude(planet_positions)
    " A - B + C "
    sastra_sagam_long = jupiter_long - saturn_long + mercury_long
    if not _is_C_between_B_to_A(jupiter_long,saturn_long,mercury_long):
        sastra_sagam_long += 30
    if night_time_birth:
        sastra_sagam_long = saturn_long - jupiter_long + mercury_long
        if not _is_C_between_B_to_A(saturn_long,jupiter_long,mercury_long):
            sastra_sagam_long += 30
    sastra_sagam_long %= 360
    return sastra_sagam_long
def bandhu_saham(planet_positions,night_time_birth=False):
# 19 Bandhu Relatives Mercury – Moon + Lagna
    mercury_long = mercury_longitude(planet_positions)
    moon_long = moon_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    bandhu_sagam_long = mercury_long - moon_long + lagna_long
    if not _is_C_between_B_to_A(mercury_long,moon_long,lagna_long):
        bandhu_sagam_long += 30
    if night_time_birth:
        bandhu_sagam_long = moon_long - mercury_long + lagna_long
        if not _is_C_between_B_to_A(moon_long,mercury_long,lagna_long):
            bandhu_sagam_long += 30
    bandhu_sagam_long %= 360
    return bandhu_sagam_long
def mrithyu_saham(planet_positions):
# 20 Mrityu Death 8th house – Moon + Lagna (same for day & night)
    lagna_long = lagna_longitude(planet_positions)
    eigth_house_long = lagna_long + (8-1)*30
    moon_long = moon_longitude(planet_positions)
    " A - B + C "
    mrithyu_sagam_long = eigth_house_long - moon_long + lagna_long
    if not _is_C_between_B_to_A(eigth_house_long,moon_long,lagna_long):
        mrithyu_sagam_long += 30
    mrithyu_sagam_long %= 360
    return mrithyu_sagam_long
def paradesa_saham(planet_positions,night_time_birth=False):
# 21 Paradesa Foreign countries 9th house – 9th lord + Lagna (same for day & night)
    h_to_p = utils.get_house_planet_list_from_planet_positions(planet_positions)
    asc_house = planet_positions[0][1][0]
    ninth_house = (asc_house+9-1)%12
    ninth_lord = house.house_owner_from_planet_positions(planet_positions,ninth_house)
    long_asc_house = lagna_longitude(planet_positions)
    long_ninth_house = long_asc_house+(9-1)*30.0
    long_ninth_lord = saham_longitude(planet_positions,ninth_lord+1)
    " A - B + C "
    paradesa_saham_long = (long_ninth_house - long_ninth_lord + long_asc_house)
    if not _is_C_between_B_to_A(long_ninth_house,long_ninth_lord,long_asc_house):
        paradesa_saham_long += 30
    paradesa_saham_long %= 360
    return paradesa_saham_long
def artha_saham(planet_positions,night_time_birth=False):
# 22 Artha Money 2nd house – 2nd lord + Lagna (same for day & night)
    h_to_p = utils.get_house_planet_list_from_planet_positions(planet_positions)
    asc_house = planet_positions[0][1][0]
    second_house = (asc_house+2-1)%12
    second_lord = house.house_owner_from_planet_positions(planet_positions,second_house)
    #print('asc_house',asc_house,'second_house',second_house,'second_lord',second_lord)
    long_asc_house = planet_positions[0][1][0]*30+planet_positions[0][1][1]
    long_second_house = long_asc_house+30.0
    long_second_lord = planet_positions[second_lord+1][1][0]*30+planet_positions[second_lord+1][1][1]
    " A - B + C "
    artha_saham_long = (long_second_house - long_second_lord + long_asc_house)
    if not _is_C_between_B_to_A(long_second_house,long_second_lord,long_asc_house):
        artha_saham_long += 30
    artha_saham_long %= 360
    return artha_saham_long
def paradara_saham(planet_positions,night_time_birth=False):
# 23 Paradara Adultery Venus – Sun + Lagna
    venus_long = venus_longitude(planet_positions)
    sun_long = sun_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    paradara_sagam_long = venus_long - sun_long + lagna_long
    if not _is_C_between_B_to_A(venus_long,sun_long,lagna_long):
        paradara_sagam_long += 30
    if night_time_birth:
        paradara_sagam_long = sun_long - venus_long + lagna_long
        if not _is_C_between_B_to_A(sun_long,venus_long,lagna_long):
            paradara_sagam_long += 30
    paradara_sagam_long %= 360
    return paradara_sagam_long
def vanika_saham(planet_positions,night_time_birth=False):
# 24 Vanik Commerce Moon – Mercury + Lagna
    moon_long = moon_longitude(planet_positions) #345-14 11-15-14
    mercury_long = mercury_longitude(planet_positions) # 311-28 - 10-11-28
    lagna_long = lagna_longitude(planet_positions) # 280-50 - 9-10-50
    " A - B + C "
    vanika_sagam_long = moon_long - mercury_long + lagna_long
    if not _is_C_between_B_to_A(moon_long,mercury_long,lagna_long):
        vanika_sagam_long += 30
    if night_time_birth:
        vanika_sagam_long = mercury_long - moon_long + lagna_long
        if not _is_C_between_B_to_A(mercury_long,moon_long,lagna_long):
            vanika_sagam_long += 30
    vanika_sagam_long %= 360
    return vanika_sagam_long
def karyasiddhi_saham(planet_positions,night_time_birth=False):
# 25 Karyasiddhi Success in endeavours Saturn – Sun + Lord of sunsign (Night: Saturn – Moon + Lord of Moonsign)
    h_to_p = utils.get_house_planet_list_from_planet_positions(planet_positions)
    saturn_long = saturn_longitude(planet_positions)
    B_long = sun_longitude(planet_positions)
    lord_of_sun_sign = house.house_owner_from_planet_positions(planet_positions,planet_positions[1][1][0])
    sign_long = saham_longitude(planet_positions,lord_of_sun_sign+1)
    " A - B + C "
    karyasiddhi_sagam_long = saturn_long - B_long + sign_long
    if not _is_C_between_B_to_A(saturn_long,B_long,sign_long):
        karyasiddhi_sagam_long += 30
    if night_time_birth:
        B_long = moon_longitude(planet_positions)
        lord_of_moon_sign = house.house_owner_from_planet_positions(planet_positions,planet_positions[2][1][0])
        sign_long = saham_longitude(planet_positions,lord_of_moon_sign+1)
        karyasiddhi_sagam_long = saturn_long - B_long + sign_long
        if not _is_C_between_B_to_A(saturn_long,B_long,sign_long):
            karyasiddhi_sagam_long += 30
    karyasiddhi_sagam_long %= 360
    return karyasiddhi_sagam_long
    
def vivaha_saham(planet_positions,night_time_birth=False):
# 26 Vivaha Marriage Venus – Saturn + Lagna
    venus_long = venus_longitude(planet_positions)
    saturn_long = saturn_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    vivaha_saham_long = venus_long - saturn_long + lagna_long 
    if not _is_C_between_B_to_A(venus_long,saturn_long,lagna_long):
        vivaha_saham_long += 30
    if night_time_birth:
        vivaha_saham_long = saturn_long - venus_long + lagna_long
        if not _is_C_between_B_to_A(saturn_long,venus_long,lagna_long):
            vivaha_saham_long += 30
    vivaha_saham_long %= 360
    return vivaha_saham_long
def santapa_saham(planet_positions,night_time_birth=False):
# 27 Santapa Sadness Saturn – Moon + 6th house
    saturn_long = saturn_longitude(planet_positions)
    moon_long = moon_longitude(planet_positions)
    sixth_house_long = lagna_longitude(planet_positions) + (6-1)*30
    " A - B + C "
    santapa_saham_long = saturn_long - moon_long + sixth_house_long 
    if not _is_C_between_B_to_A(saturn_long,moon_long,sixth_house_long):
        santapa_saham_long += 30
    if night_time_birth:
        santapa_saham_long = moon_long - saturn_long + sixth_house_long
        if not _is_C_between_B_to_A(moon_long,saturn_long,sixth_house_long):
            santapa_saham_long += 30
    santapa_saham_long %= 360
    return santapa_saham_long
def sraddha_saham(planet_positions,night_time_birth=False):
# 28 Sraddha Devotion/sincerity Venus – Mars + Lagna
    venus_long = venus_longitude(planet_positions)
    mars_long = mars_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    sraddha_sagam_long = venus_long - mars_long + lagna_long
    if not _is_C_between_B_to_A(venus_long,mars_long,lagna_long):
        sraddha_sagam_long += 30
    if night_time_birth:
        sraddha_sagam_long = mars_long - venus_long + lagna_long
        if not _is_C_between_B_to_A(mars_long,venus_long,lagna_long):
            sraddha_sagam_long += 30
    sraddha_sagam_long %= 360
    return sraddha_sagam_long
def preethi_saham(planet_positions,night_time_birth=False):
# 29 Preeti Love/attachment SastraSaham – PunyaSaham + Lagna
    sastra_long = sastra_saham(planet_positions, night_time_birth)
    punya_long = punya_saham(planet_positions, night_time_birth)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    preethi_sagam_long = sastra_long - punya_long + lagna_long
    if not _is_C_between_B_to_A(sastra_long,punya_long,lagna_long):
        preethi_sagam_long += 30
    if night_time_birth:
        preethi_sagam_long = punya_long - sastra_long + lagna_long
        if not _is_C_between_B_to_A(punya_long,sastra_long,lagna_long):
            preethi_sagam_long += 30
    preethi_sagam_long %= 360
    return preethi_sagam_long
def jadya_saham(planet_positions,night_time_birth=False):
# 30 Jadya Chronic disease Mars – Saturn + Mercury
    mars_long = mars_longitude(planet_positions)
    saturn_long = saturn_longitude(planet_positions)
    mercury_long = mercury_longitude(planet_positions)
    " A - B + C "
    jadya_sagam_long = mars_long - saturn_long + mercury_long
    if not _is_C_between_B_to_A(mars_long,saturn_long,mercury_long):
        jadya_sagam_long += 30
    if night_time_birth:
        jadya_sagam_long = saturn_long - mars_long + mercury_long
        if not _is_C_between_B_to_A(saturn_long,mars_long,mercury_long):
            jadya_sagam_long += 30
        jadya_sagam_long %= 360
    return jadya_sagam_long
def vyaapaara_saham(planet_positions):
# 31 Vyapara Business Mars – Saturn + Lagna (same for day & night)
    mars_long = mars_longitude(planet_positions)
    saturn_long = saturn_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    vyaapaara_sagam_long = mars_long - saturn_long + lagna_long
    if not _is_C_between_B_to_A(mars_long,saturn_long,lagna_long):
        vyaapaara_sagam_long += 30
    vyaapaara_sagam_long %= 360
    return vyaapaara_sagam_long
def sathru_saham(planet_positions,night_time_birth=False):
# 32 Satru Enemy Mars – Saturn + Lagna
    mars_long = mars_longitude(planet_positions)
    saturn_long = saturn_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    sathru_sagam_long = mars_long - saturn_long + lagna_long
    if not _is_C_between_B_to_A(mars_long,saturn_long,lagna_long):
        sathru_sagam_long += 30
    if night_time_birth:
        sathru_sagam_long = saturn_long - mars_long + lagna_long
        if not _is_C_between_B_to_A(saturn_long,mars_long,lagna_long):
            sathru_sagam_long += 30
    sathru_sagam_long %= 360
    return sathru_sagam_long
def jalapatna_saham(planet_positions,night_time_birth=False):
# 33 Jalapatana Crossing an ocean Cancer 15º– Saturn + Lagna
    cancer_long = 105.0
    saturn_long = saturn_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    jalapatna_sagam_long = cancer_long - saturn_long + lagna_long
    if not _is_C_between_B_to_A(cancer_long,saturn_long,lagna_long):
        jalapatna_sagam_long += 30
    if night_time_birth:
        jalapatna_sagam_long = saturn_long - cancer_long + lagna_long
        if not _is_C_between_B_to_A(saturn_long,cancer_long,lagna_long):
            jalapatna_sagam_long += 30
    jalapatna_sagam_long %= 360
    return jalapatna_sagam_long
def bandhana_saham(planet_positions,night_time_birth=False):
# 34 Bandhana Imprisonment PunyaSaham – Saturn + Lagna
    punya_long = punya_saham(planet_positions, night_time_birth)
    saturn_long = saturn_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    bandhana_sagam_long = punya_long - saturn_long + lagna_long
    if not _is_C_between_B_to_A(punya_long,saturn_long,lagna_long):
        bandhana_sagam_long += 30
    if night_time_birth:
        bandhana_sagam_long = saturn_long - punya_long + lagna_long
        if not _is_C_between_B_to_A(saturn_long,punya_long,lagna_long):
            bandhana_sagam_long += 30
    bandhana_sagam_long %= 360
    return bandhana_sagam_long
def apamrithyu_saham(planet_positions,night_time_birth=False):
# 35 Apamrityu Bad death 8th house – Mars + Lagna
    eigth_house_long = lagna_longitude(planet_positions)+210
    mars_long = mars_longitude(planet_positions)
    lagna_long = lagna_longitude(planet_positions)
    " A - B + C "
    apamrithyu_sagam_long = eigth_house_long - mars_long + lagna_long
    if not _is_C_between_B_to_A(eigth_house_long,mars_long,lagna_long):
        apamrithyu_sagam_long += 30
    if night_time_birth:
        apamrithyu_sagam_long = mars_long - eigth_house_long + lagna_long
        if not _is_C_between_B_to_A(mars_long,eigth_house_long,lagna_long):
            apamrithyu_sagam_long += 30
    apamrithyu_sagam_long %= 360
    return apamrithyu_sagam_long
def laabha_saham(planet_positions,night_time_birth=False):
# 36 Labha Material gains 11th house – 11th lord + Lagna (same for day & night)
    h_to_p = utils.get_house_planet_list_from_planet_positions(planet_positions)
    asc_house = planet_positions[0][1][0]
    eleventh_house = (asc_house+11-1)%12
    eleventh_lord = house.house_owner_from_planet_positions(planet_positions,eleventh_house)
    long_asc_house = lagna_longitude(planet_positions)
    long_eleventh_house = long_asc_house+(11-1)*30.0
    long_eleventh_lord = saham_longitude(planet_positions,eleventh_lord+1)
    " A - B + C "
    laabha_saham_long = (long_eleventh_house - long_eleventh_lord + long_asc_house)
    if not _is_C_between_B_to_A(long_eleventh_house,long_eleventh_lord,long_asc_house):
        laabha_saham_long += 30
    if night_time_birth:
        laabha_saham_long = (long_eleventh_lord - long_eleventh_house + long_asc_house)
        if not _is_C_between_B_to_A(long_eleventh_lord,long_eleventh_house,long_asc_house):
            laabha_saham_long += 30
    laabha_saham_long %= 360
    return laabha_saham_long
def _is_C_between_B_to_A(a_long,b_long,c_long):
    a_rasi = int(a_long/30)
    b_rasi = int(b_long/30)
    c_rasi = int(c_long/30)
    c_rasi_found = False
    for n in range(b_rasi,b_rasi+11):
        next_n = (n+1) % 12 
        if next_n == c_rasi:
            c_rasi_found = True
            break
        elif next_n == a_rasi:
            break
    return c_rasi_found
     
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests.saham_tests()
    exit()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Calculates Ashtottari (=108) Dasha-bhukthi-antara-sukshma-prana
"""

from collections import OrderedDict as Dict
from jhora import const,utils
from jhora.panchanga import drik
from jhora.horoscope.chart import house
year_duration = const.sidereal_year# const.tropical_year  # some say 360 days, others 365.25 or 365.2563 etc
human_life_span_for_ashtottari_dhasa = 108
""" 
    {ashtottari adhipati:[(starting_star_number,ending_star_number),dasa_length]} 
        ashtottari longitude range: (starting_star_number-1) * 360/27 TO (ending_star_number) * 360/27
        Example: 66.67 to 120.00 = 53 deg 20 min range
"""
ashtottari_adhipathi_list = [0,1,2,3,6,4,7,5]
ashtottari_adhipathi_dict_seed = {0:[(6,9),6],1:[(10,12),15],2:[(13,16),8],3:[(17,19),17],
                             6:[(20,22),10],4:[(23,25),19],7:[(26,2),12],5:[(3,5),21]}
def applicability_check(planet_positions):
    asc_house = planet_positions[0][1][0]
    lagna_lord = house.house_owner_from_planet_positions(planet_positions, asc_house)
    house_of_lagna_lord = planet_positions[lagna_lord+1][1][0]
    rahu_house = planet_positions[8][1][0]
    chk1 =  rahu_house in house.trines_of_the_raasi(house_of_lagna_lord) and rahu_house != asc_house
    chk2 =  rahu_house in house.quadrants_of_the_raasi(house_of_lagna_lord) and rahu_house != asc_house 
    return chk1 or chk2
def _get_dhasa_dict(seed_star=6):
    if seed_star==6: return ashtottari_adhipathi_dict_seed
    ashtottari_adhipathi_dict = {}
    nak = seed_star
    for p,[(nb,ne),durn] in ashtottari_adhipathi_dict_seed.items():
        nak_diff = ne-nb
        nsb = nak; nse = (nsb + nak_diff)%28
        ashtottari_adhipathi_dict[p] = [(nsb,nse),durn]
        nak = (nse+1)%28
    return ashtottari_adhipathi_dict
def ashtottari_adhipathi(nak):
    for key,value in ashtottari_adhipathi_dict.items():
        starting_star = value[0][0]
        ending_star = value[0][1]
        nak1 = nak
        if ending_star < starting_star:# and nak < starting_star:
            ending_star += 27
            if nak1 < starting_star:
                nak1 += 27
        if nak1 >= starting_star and nak1 <= ending_star:
            return key,value
def ashtottari_dasha_start_date(jd,place,divisional_chart_factor=1,chart_method=1,star_position_from_moon=1,
                                dhasa_starting_planet=1):
    y,m,d,fh = utils.jd_to_gregorian(jd); dob=drik.Date(y,m,d); tob=(fh,0,0)
    one_star = (360 / 27.)        # 27 nakshatras span 360°
    from jhora.horoscope.chart import charts,sphuta
    _special_planets = ['M','G','T','I','B','I','P']
    planet_positions = charts.divisional_chart(jd, place, divisional_chart_factor=divisional_chart_factor,
                                               chart_method=chart_method)
    if dhasa_starting_planet in [*range(9)]:
        planet_long = planet_positions[dhasa_starting_planet+1][1][0]*30+planet_positions[dhasa_starting_planet+1][1][1]
    elif dhasa_starting_planet==const._ascendant_symbol:
        planet_long = planet_positions[0][1][0]*30+planet_positions[0][1][1]
    elif dhasa_starting_planet.upper()=='M':
        mn = drik.maandi_longitude(dob,tob,place,divisional_chart_factor=divisional_chart_factor)
        planet_long = mn[0]*30+mn[1]
    elif dhasa_starting_planet.upper()=='G':
        gl = drik.gulika_longitude(dob,tob,place,divisional_chart_factor=divisional_chart_factor)
        planet_long = gl[0]*30+gl[1]
    elif dhasa_starting_planet.upper()=='B':
        gl = drik.bhrigu_bindhu_lagna(jd, place,divisional_chart_factor=divisional_chart_factor,chart_method=chart_method)
        planet_long = gl[0]*30+gl[1]
    elif dhasa_starting_planet.upper()=='I':
        gl = drik.indu_lagna(jd, place,divisional_chart_factor=divisional_chart_factor,chart_method=chart_method)
        planet_long = gl[0]*30+gl[1]
    elif dhasa_starting_planet.upper()=='P':
        gl = drik.pranapada_lagna(jd, place,divisional_chart_factor=divisional_chart_factor)
        planet_long = gl[0]*30+gl[1]
    elif dhasa_starting_planet.upper()=='T':
        sp = sphuta.tri_sphuta(dob,tob,place,divisional_chart_factor=divisional_chart_factor,chart_method=chart_method)
        planet_long = sp[0]*30+sp[1]
    else:
        planet_long = planet_positions[2][1][0]*30+planet_positions[2][1][1]
    if dhasa_starting_planet==1:
        planet_long += (star_position_from_moon-1)*one_star
    nak = int(planet_long / one_star)
    lord,res = ashtottari_adhipathi(nak+1)          # ruler of current nakshatra
    period = res[1]; start_nak = res[0][0]; end_nak = res[0][1]
    period_elapsed = ( planet_long - (start_nak-1)*one_star)/((end_nak-start_nak+1)*one_star)
    period_elapsed *= (period*year_duration)        # days
    start_date = jd - period_elapsed      # so many days before current day
    return [lord, start_date]
def ashtottari_next_adhipati(lord,dirn=1):
    """Returns next lord after `lord` in the adhipati_list"""
    current = ashtottari_adhipathi_list.index(lord)
    #print(current)
    next_index = (current + dirn) % len(ashtottari_adhipathi_list)
    #print(next_index)
    return ashtottari_adhipathi_list[next_index]
def ashtottari_mahadasa(jd,place,divisional_chart_factor=1,chart_method=1,star_position_from_moon=1,
                        dhasa_starting_planet=1):
    """
        returns a dictionary of all mahadashas and their start dates
        @return {mahadhasa_lord_index, (starting_year,starting_month,starting_day,starting_time_in_hours)}
    """
    lord, start_date = ashtottari_dasha_start_date(jd,place,divisional_chart_factor=divisional_chart_factor,
                                chart_method=chart_method,star_position_from_moon=star_position_from_moon,
                                dhasa_starting_planet=dhasa_starting_planet)
    retval = Dict()
    for _ in range(len(ashtottari_adhipathi_list)):
        retval[lord] = start_date
        lord_duration = ashtottari_adhipathi_dict[lord][1]
        start_date += lord_duration * year_duration
        lord = ashtottari_next_adhipati(lord)
    return retval
def ashtottari_bhukthi(dhasa_lord, start_date,antardhasa_option=1):
    """
        Compute all bhukthis of given nakshatra-lord of Mahadasa and its start date
    """
    lord = dhasa_lord
    if antardhasa_option in [3,4]:
        lord = ashtottari_next_adhipati(dhasa_lord, dirn=1) 
    elif antardhasa_option in [5,6]:
        lord = ashtottari_next_adhipati(dhasa_lord, dirn=-1) 
    dirn = 1 if antardhasa_option in [1,3,5] else -1
    retval = Dict()
    #lord = dhasa_lord if const.ashtottari_bhukthi_starts_from_dhasa_lord else ashtottari_next_adhipati(dhasa_lord)
    dhasa_lord_duration = ashtottari_adhipathi_dict[lord][1]
    for _ in range(len(ashtottari_adhipathi_list)):
        retval[lord] = start_date
        lord_duration = ashtottari_adhipathi_dict[lord][1]
        factor = lord_duration * dhasa_lord_duration / human_life_span_for_ashtottari_dhasa
        start_date += factor * year_duration
        lord = ashtottari_next_adhipati(lord,dirn)
    return retval
def ashtottari_anthara(dhasa_lord, bhukthi_lord,bhukthi_lord_start_date):
    """
        Compute all bhukthis of given nakshatra-lord of Mahadasa, its bhukthi lord and bhukthi_lord's start date
    """
    dhasa_lord_duration = ashtottari_adhipathi_dict[dhasa_lord][1]
    retval = Dict()
    lord = bhukthi_lord# if const.ashtottari_bhukthi_starts_from_dhasa_lord else ashtottari_next_adhipati(bhukthi_lord)
    for i in range(len(ashtottari_adhipathi_list)):
        retval[lord] = bhukthi_lord_start_date
        lord_duration = ashtottari_adhipathi_dict[lord][1]
        factor = lord_duration * dhasa_lord_duration / human_life_span_for_ashtottari_dhasa
        bhukthi_lord_start_date += factor * year_duration
        lord = ashtottari_next_adhipati(lord)
    return retval
def get_ashtottari_dhasa_bhukthi(jd, place,divisional_chart_factor=1,chart_method=1,star_position_from_moon=1,
                                 use_tribhagi_variation=False,include_antardhasa=True,
                                 antardhasa_option=1,dhasa_starting_planet=1,seed_star=6):
    """
        provides Ashtottari dhasa bhukthi for a given date in julian day (includes birth time)
        @param jd: Julian day for birthdate and birth time
        @param place: Place as tuple (place name, latitude, longitude, timezone) 
        @param divisional_chart_factor Default=1 
            1=Raasi, 9=Navamsa. See const.division_chart_factors for options
        @param chart_method: Default 1; various methods available for each divisional chart. See charts module
        @param star_position_from_moon: 
            1 => Default - moon
            4 => Kshema Star (4th constellation from moon)
            5 => Utpanna Star (5th constellation from moon)
            8 => Adhana Star (8th constellation from moon)
        @param use_tribhagi_variation: False (default), True means dhasa bhukthi duration in three phases 
        @param include_antardhasa: True (include) False (exclude) antardhasa (Default=True)
        @param antardhasa_option:
            1 => dhasa lord - forward (Default)
            2 => dhasa lord - backward
            3 => next dhasa lord - forward
            4 => next dhasa lord - backward
            5 => prev dhasa lord - forward
            6 => prev dhasa lord - backward
        @param dhasa_starting_planet 0=Sun 1=Moon(default)...8=Ketu, 'L'=Lagna
                                    M=Maandi, G=Gulika, T=Trisphuta, B=Bhindu, I=Indu, P=Pranapada
        @param seed_star 1..27. Default = 6
        @return: a list of [dhasa_lord,bhukthi_lord,bhukthi_start]
          Example: [ [7, 5, '1915-02-09'], [7, 0, '1917-06-10'], [7, 1, '1918-02-08'],...]
    """
    global human_life_span_for_ashtottari_dhasa, ashtottari_adhipathi_dict
    ashtottari_adhipathi_dict = _get_dhasa_dict(seed_star)
    _dhasa_cycles = 1
    _tribhagi_factor = 1
    if use_tribhagi_variation:
        _tribhagi_factor = 1./3.
        _dhasa_cycles = int(_dhasa_cycles/_tribhagi_factor)
        human_life_span_for_ashtottari_dhasa *= _tribhagi_factor
        for k,(v1,v2) in ashtottari_adhipathi_dict.items():
            ashtottari_adhipathi_dict[k] = [v1,v2*_tribhagi_factor]#[v1,round(v2*_tribhagi_factor,2)]
    dashas = ashtottari_mahadasa(jd,place,divisional_chart_factor=divisional_chart_factor,
                                 star_position_from_moon=star_position_from_moon,
                                 dhasa_starting_planet=dhasa_starting_planet)
    dhasa_bhukthi=[]
    for _ in range(_dhasa_cycles):
        for i in dashas:
            dhasa_lord = i
            if include_antardhasa:
                bhukthis = ashtottari_bhukthi(i, dashas[i],antardhasa_option)
                for j in bhukthis:
                    bhukthi_lord = j
                    jd1 = bhukthis[j]
                    y, m, d, h = utils.jd_to_gregorian(jd1)#swe.revjul(round(jd1 + tz/24))
                    """ TODO: Need to figure out passing date and time string to UI, main.py and pvr_tests.py """
                    date_str = '%04d-%02d-%02d' %(y,m,d)+' '+utils.to_dms(h,as_string=True)
                    dhasa_bhukthi.append([dhasa_lord,bhukthi_lord,date_str]) 
            else:
                jd1 = dashas[i]
                y, m, d, h = utils.jd_to_gregorian(jd1)#swe.revjul(round(jd1 + tz/24))
                """ TODO: Need to figure out passing date and time string to UI, main.py and pvr_tests.py """
                date_str = '%04d-%02d-%02d' %(y,m,d)+' '+utils.to_dms(h,as_string=True)
                dhasa_bhukthi.append([dhasa_lord,date_str])                 
    return dhasa_bhukthi
'------ main -----------'
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests._STOP_IF_ANY_TEST_FAILED = False
    pvr_tests.ashtottari_tests()
    


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from PyQt6.QtWidgets import QLineEdit, QApplication, QLabel, QHBoxLayout, QVBoxLayout, QPushButton,\
                            QComboBox, QDialog, QCheckBox
from PyQt6.QtCore import Qt
from jhora import utils, const
_chart_names = ['raasi_str', 'hora_str', 'drekkanam_str', 'chaturthamsa_str', 'panchamsa_str', 
    'shashthamsa_str', 'saptamsam_str', 'ashtamsa_str', 'navamsam_str', 'dhasamsam_str', 'rudramsa_str', 
    'dhwadamsam_str', 'shodamsa_str', 'vimsamsa_str', 'chaturvimsamsa_str', 'nakshatramsa_str', 'thrisamsam_str', 
    'khavedamsa_str', 'akshavedamsa_str', 'sashtiamsam_str', 
    'nava_navamsa_str', 'ashtotharamsa_str', 'dwadas_dwadasamsa_str']
_graha_dhasa_list = ['vimsottari','yoga_vimsottari','rasi_bhukthi_vimsottari','ashtottari','tithi_ashtottari',
                     'yogini','tithi_yogini','shodasottari','dwadasottari','dwisatpathi','panchottari','satabdika',
                     'chaturaaseeti_sama','karana_chaturaaseeti_sama','shashtisama','shattrimsa_sama','naisargika',
                     'tara','karaka','buddhi_gathi','kaala','aayu','saptharishi_nakshathra']
_tithi_list = ['janma_tithi_str','dhana_tithi_str','bhrartri_tithi_str','matri_tithi_str','putra_tithi_str',
               'satru_tithi_str','kalatra_tithi_str','mrutyu_tithi_str','bhagya_tithi_str','karma_tithi_str',
               'laabha_tithi_str','vyaya_tithi_str']
seed_list_dhasas = {0:3,2:3,3:6,5:7,7:8,8:27,9:19,10:17,11:27,12:15,14:1,15:22}
dhasa_start_planet_dhasas = [0,2,3,5,7,8,9,10,11,12,14,15,22]
start_from_moon_dhasas = [0,2,3,5,7,8,9,10,11,12,14,15,22]
tribhagi_dhasas = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,22]
varga_dhasas = [0,2,3,5,7,8,9,10,11,12,14,15,17,19,22]
antardhasa_option_list = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,22]
tithi_dhasas = [4,6]
_varga_option_dict = const.varga_option_dict
class DhasaBhukthiOptionDialog(QDialog):
    """
        Dhasa Bhukthi Options Dialog
    """
    def __init__(self, dhasa_index=0, dhasa_option_list=None):
        super().__init__()
        #utils.set_language(const._DEFAULT_LANGUAGE)
        self._dhasa_index = dhasa_index
        self._dhasa_option_list = const.dhasa_default_options[dhasa_index] if dhasa_option_list==None else dhasa_option_list
        self.res = utils.resource_strings
        self._graha_dhasa_dict = [self.res[d+'_str'] for d in _graha_dhasa_list]
        self._dhasa_name = self._graha_dhasa_dict[self._dhasa_index]
        self._tribhagi_visible = False
        self._aayu_visible = False
        self._seed_star_visible = False
        self._dhasa_start_visible = False
        self._moon_start_visible = False
        self._ant_option_visible = False
        self._varga_chart_visible = False
        self._varga_chart_method_visible = False
        self._naisargika_option_visible = False
        self._tithi_list_visible = False
        self._rbv_variation_visible = False
        self._tara_method_visible = False
        self._option_string = ''
        self.create_ui()
    def closeEvent(self, *args, **kwargs):
        self._option_string = ''
        QApplication.restoreOverrideCursor()
        return QDialog.closeEvent(self, *args, **kwargs)
    def create_ui(self):
        v_layout = QVBoxLayout()
        title = self._dhasa_name+' '+self.res['dhasa_str']+' '+self.res['options_str']
        self.setWindowTitle(title)
        _option_counter = 0
        if self._dhasa_index==21: #'aayu':
            self._aayu_visible = True
            h_layout1 = QHBoxLayout()
            _label = QLabel(self.res['aayu_dhasa_option_str'])
            h_layout1.addWidget(_label)
            dhasa_type_list = [self.res['pindayu_str'],self.res['nisargayu_str'],self.res['amsayu_str'],'None']
            self._dhasa_type_combo = QComboBox()
            self._dhasa_type_combo.addItems(dhasa_type_list)
            self._dhasa_type_combo.setCurrentIndex(self._dhasa_option_list[_option_counter]); _option_counter+=1
            h_layout1.addWidget(self._dhasa_type_combo)
            v_layout.addLayout(h_layout1)
            self.setLayout(v_layout)
        elif self._dhasa_index==16: # Naisargika
            self._naisargika_option_visible = True
            v_layout1 = QVBoxLayout()
            _label = QLabel(self.res['nisargika_dhasa_option1_str'])
            v_layout1.addWidget(_label)
            self._naisargika_option1 = QCheckBox(self.res['nisargika_dhasa_option2_str'])
            self._naisargika_option1.setChecked(self._dhasa_option_list[_option_counter]); _option_counter+=1
            v_layout1.addWidget(self._naisargika_option1)
            self._naisargika_option2 = QCheckBox(self.res['nisargika_dhasa_option3_str'])
            self._naisargika_option2.setChecked(self._dhasa_option_list[_option_counter]); _option_counter+=1
            v_layout1.addWidget(self._naisargika_option2)
            self._naisargika_option3 = QCheckBox(self.res['nisargika_dhasa_option4_str'])
            self._naisargika_option3.setChecked(self._dhasa_option_list[_option_counter]); _option_counter+=1
            v_layout1.addWidget(self._naisargika_option3)
            v_layout.addLayout(v_layout1)
            self.setLayout(v_layout)
        elif self._dhasa_index==17: # Tara
            self._tara_method_visible = True
            _label = QLabel(self.res['tara_dhasa_option1_str'])
            v_layout.addWidget(_label)
            self._tara_method_combo = QComboBox()
            self._tara_method_combo.addItems([self.res['tara_dhasa_option2_str'],
                                              self.res['tara_dhasa_option3_str']])
            self._tara_method_combo.setCurrentIndex(self._dhasa_option_list[_option_counter]); _option_counter+=1
            v_layout.addWidget(self._tara_method_combo)
            self.setLayout(v_layout)
        elif self._dhasa_index==2: # Raasi Bhukthi Variation
            self._rbv_variation_visible = True
            self._rbv_check = QCheckBox(self.res['rbv_dhasa_option_str'])
            self._rbv_check.setChecked(self._dhasa_option_list[_option_counter]); _option_counter+=1
            self._rbv_check.setEnabled(False)
            v_layout.addWidget(self._rbv_check)
            self.setLayout(v_layout)
        if self._dhasa_index in tithi_dhasas:
            self._tithi_list_visible = True
            h_layout1 = QHBoxLayout()
            _label = QLabel(self.res['tithi_index_option_str'])
            h_layout1.addWidget(_label)
            self._tithi_combo = QComboBox()
            self._tithi_combo.addItems([self.res[t] for t in _tithi_list])
            self._tithi_combo.setCurrentIndex(self._dhasa_option_list[_option_counter]); _option_counter+=1
            h_layout1.addWidget(self._tithi_combo)
            v_layout.addLayout(h_layout1)
        if self._dhasa_index in tribhagi_dhasas: # Tribhagi
            self._tribhagi_visible = True
            h_layout1 = QHBoxLayout()
            _label = QLabel(self.res['tribhagi_option_str'])
            h_layout1.addWidget(_label)
            self._tribhagi_check = QCheckBox()
            self._tribhagi_check.setChecked(self._dhasa_option_list[_option_counter]); _option_counter+=1
            h_layout1.addWidget(self._tribhagi_check)
            v_layout.addLayout(h_layout1)
        if self._dhasa_index in seed_list_dhasas.keys(): # Seed Star
            self._seed_star_visible = True
            h_layout1 = QHBoxLayout()
            _label = QLabel(self.res['seed_star_option_str'])
            h_layout1.addWidget(_label)
            self._seed_star_combo = QComboBox()
            seed_list = [utils.NAKSHATRA_LIST[n] for n in range(27)]
            self._seed_star_combo.addItems(seed_list)
            self._seed_star_combo.setCurrentIndex(self._dhasa_option_list[_option_counter]); _option_counter+=1
            h_layout1.addWidget(self._seed_star_combo)
            v_layout.addLayout(h_layout1)
        if self._dhasa_index in dhasa_start_planet_dhasas: # dhasa start
            self._dhasa_start_visible = True
            h_layout1 = QHBoxLayout()
            _label = QLabel(self.res['dhasa_start_option_str'])
            h_layout1.addWidget(_label)
            self._dhasa_start_combo = QComboBox()
            dhasa_start_list = [utils.PLANET_NAMES[p]+'-'+self.res['sphuta_str'] for p in range(9)]+\
                    [self.res['ascendant_str']+'-'+self.res['sphuta_str'],self.res['maandi_str']+'-'+self.res['sphuta_str'],
                    self.res['gulika_str']+'-'+self.res['sphuta_str'],self.res['tri_sphuta_str']+'-'+self.res['sphuta_str'],
                    self.res['bhrigu_bindhu_lagna_str'],self.res['indu_lagna_str'],self.res['pranapada_lagna_str']]
            self._dhasa_start_combo.addItems(dhasa_start_list)
            self._dhasa_start_combo.setCurrentIndex(self._dhasa_option_list[_option_counter]); _option_counter+=1
            h_layout1.addWidget(self._dhasa_start_combo)
            v_layout.addLayout(h_layout1)
        if self._dhasa_index in start_from_moon_dhasas: # start from moon
            self._moon_start_visible = True
            h_layout1 = QHBoxLayout()
            _label = QLabel(self.res['moon_start_option1_str'])
            h_layout1.addWidget(_label)
            self._moon_start_combo = QComboBox()
            moon_start_list = [self.res['moon_start_option2_str'],self.res['moon_start_option3_str'],
                                self.res['moon_start_option4_str'],self.res['moon_start_option5_str']]
            self._moon_start_combo.addItems(moon_start_list)
            self._moon_start_combo.setCurrentIndex(self._dhasa_option_list[_option_counter]); _option_counter+=1
            h_layout1.addWidget(self._moon_start_combo)
            v_layout.addLayout(h_layout1)
        if self._dhasa_index in antardhasa_option_list: # Antardhasa Option
            self._ant_option_visible = True
            h_layout1 = QHBoxLayout()
            _label = QLabel(self.res['ant_dhasa_option_str'])
            h_layout1.addWidget(_label)
            self._ant_option_combo = QComboBox()
            ant_option_list = [self.res['ant_dhasa_option1_str'],self.res['ant_dhasa_option2_str'],
                               self.res['ant_dhasa_option3_str'],self.res['ant_dhasa_option4_str'],
                               self.res['ant_dhasa_option5_str'],self.res['ant_dhasa_option6_str']]
            self._ant_option_combo.addItems(ant_option_list)
            self._ant_option_combo.setCurrentIndex(self._dhasa_option_list[_option_counter]); _option_counter+=1
            h_layout1.addWidget(self._ant_option_combo)
            v_layout.addLayout(h_layout1)
        if self._dhasa_index in varga_dhasas: # Varga Chart 
            self._varga_chart_visible = True
            h_layout1 = QHBoxLayout()
            _label = QLabel(self.res['varga_option_str'])
            h_layout1.addWidget(_label)
            self._varga_chart_combo = QComboBox()
            varga_chart_list = [self.res[c] for c in _chart_names]
            self._varga_chart_combo.addItems(varga_chart_list)
            self._varga_chart_combo.setCurrentIndex(self._dhasa_option_list[_option_counter]); _option_counter+=1
            self._varga_chart_combo.currentIndexChanged.connect(self._varga_chart_selection_changed)
            h_layout1.addWidget(self._varga_chart_combo)
            v_layout.addLayout(h_layout1)
            if self._dhasa_option_list[_option_counter] != -1:
                h_layout1 = QHBoxLayout()
                self._varga_chart_method_label = QLabel(self.res['choose_varga_chart_method_str'])
                self._varga_chart_method_label.setVisible(True)
                h_layout1.addWidget(self._varga_chart_method_label)
                self._varga_chart_method_combo = QComboBox()
                self._varga_chart_method_combo.setVisible(True)
                _varga_chart_index = self._varga_chart_combo.currentIndex()
                _varga_factor = const.division_chart_factors[_varga_chart_index]
                _method_count = _varga_option_dict[_varga_factor][0]
                for m in range(_method_count):
                    _varga_method = self.res['d'+str(_varga_factor)+'_option'+str(m+1)+'_str']
                    self._varga_chart_method_combo.addItem(_varga_method)
                self._varga_chart_method_combo.setCurrentIndex(self._dhasa_option_list[_option_counter]); _option_counter+=1
                h_layout1.addWidget(self._varga_chart_method_combo)
                v_layout.addLayout(h_layout1)
            else:
                h_layout1 = QHBoxLayout()
                self._varga_chart_method_label = QLabel(self.res['choose_varga_chart_method_str'])
                self._varga_chart_method_label.setVisible(False)
                h_layout1.addWidget(self._varga_chart_method_label)
                self._varga_chart_method_combo = QComboBox()
                self._varga_chart_method_combo.setVisible(False)
                h_layout1.addWidget(self._varga_chart_method_combo)
                v_layout.addLayout(h_layout1)
        if all([self._tribhagi_visible, self._aayu_visible, self._ant_option_visible, self._dhasa_start_visible,
                self._moon_start_visible, self._seed_star_visible, self._varga_chart_visible,
                self._naisargika_option_visible, self._tithi_list_visible,self._rbv_variation_visible,
                self._tara_method_visible])==False:
            self._cancel_button_clicked()
        h_layout = QHBoxLayout()
        self._accept_button = QPushButton(self.res['accept_str'])
        self._accept_button.clicked.connect(self._accept_button_clicked)
        h_layout.addWidget(self._accept_button)
        self._cancel_button = QPushButton(self.res['cancel_str'])
        self._cancel_button.clicked.connect(self._cancel_button_clicked)
        h_layout.addWidget(self._cancel_button)
        v_layout.addLayout(h_layout)
        self.setLayout(v_layout)
    def _varga_chart_selection_changed(self):
        _varga_chart_index = self._varga_chart_combo.currentIndex()
        _varga_factor = const.division_chart_factors[_varga_chart_index]
        if _varga_factor ==1:
            self._varga_chart_method_combo.setVisible(False)
            self._varga_chart_method_visible = False
            self._varga_chart_method_label.setVisible(False)
            return
        _method_count = _varga_option_dict[_varga_factor][0]
        _default_method = _varga_option_dict[_varga_factor][1]
        self._varga_chart_method_combo.clear()
        for m in range(_method_count):
            _varga_method = self.res['d'+str(_varga_factor)+'_option'+str(m+1)+'_str']
            self._varga_chart_method_combo.addItem(_varga_method)
        self._varga_chart_method_combo.setCurrentIndex(_default_method-1)
        self._varga_chart_method_combo.setVisible(True)
        self._varga_chart_method_label.setVisible(True)
        self._varga_chart_method_visible = True
    def _accept_button_clicked(self):
        option_str = []; option_list = []
        if self._aayu_visible:
            ind = self._dhasa_type_combo.currentIndex()
            opt = 'aayur_type='+str(ind) if ind in [0,1,2] else 'aayur_type=None'
            option_str.append(opt); option_list.append(ind)
        elif self._naisargika_option_visible:
            ind1 = self._naisargika_option1.isChecked()
            opt = 'mahadhasa_lord_has_no_antardhasa='+str(bool(ind1))
            option_str.append(opt); option_list.append(bool(ind1))
            ind2 = self._naisargika_option2.isChecked()
            opt = 'antardhasa_option1='+str(bool(ind2))
            option_str.append(opt); option_list.append(bool(ind2))
            ind3 = self._naisargika_option3.isChecked()
            opt = 'antardhasa_option2='+str(bool(ind3))
            option_str.append(opt); option_list.append(bool(ind3))
        if self._tara_method_visible:
            ind = self._tara_method_combo.currentIndex()
            opt = 'dhasa_method='+str(ind+1)
            option_str.append(opt); option_list.append(ind)
        if self._rbv_variation_visible: #RBV
            ind = self._rbv_check.isChecked()
            opt = 'use_rasi_bhukthi_variation='+str(bool(ind))
            option_str.append(opt);option_list.append(bool(ind))
        if self._tithi_list_visible:
            ind = self._tithi_combo.currentIndex()
            opt = 'tithi_index='+str(ind+1)
            option_str.append(opt); option_list.append(ind)
        if self._tribhagi_visible:
            ind = self._tribhagi_check.isChecked()
            opt = 'use_tribhagi_variation='+str(bool(ind))
            option_str.append(opt); option_list.append(bool(ind))
        if self._seed_star_visible:
            ind = self._seed_star_combo.currentIndex()
            opt = 'seed_star='+str(ind+1)
            option_str.append(opt); option_list.append(ind)
        if self._dhasa_start_visible:
            ind = self._dhasa_start_combo.currentIndex()
            ind_dict = {0:'0',1:'1',2:'2',3:'3',4:'4',5:'5',6:'6',7:'7',8:'8',9:"'L'",10:"'M'",11:"'G'",12:"'T'",13:"'B'",14:"'I'",15:"'P'"}
            opt = 'dhasa_starting_planet='+str(ind_dict[ind])
            option_str.append(opt); option_list.append(ind)
        if self._moon_start_visible:
            ind = self._moon_start_combo.currentIndex()
            ind_dict = {0:1,1:4,2:5,3:8}
            opt = 'star_position_from_moon='+str(ind_dict[ind])
            option_str.append(opt); option_list.append(ind)
        if self._ant_option_visible:
            ind = self._ant_option_combo.currentIndex()
            opt = 'antardhasa_option='+str(ind+1)
            option_str.append(opt); option_list.append(ind)
        if self._varga_chart_visible:
            ind = self._varga_chart_combo.currentIndex()
            dcf = const.division_chart_factors[ind]
            opt = 'divisional_chart_factor='+str(dcf)
            option_str.append(opt); option_list.append(ind)
            if self._varga_chart_method_visible:
                ind = self._varga_chart_method_combo.currentIndex()
                opt = 'chart_method='+str(ind+1)
                option_str.append(opt); option_list.append(ind)
            else:
                option_list.append(-1)
        option_str = ','.join(option_str)
        self._accept_clicked = True
        self._option_string = option_str
        #print(option_str)
        self._option_list = option_list
        #print(option_list)
        self.accept()
    def _cancel_button_clicked(self):
        self._accept_clicked = False
        self._option_string = ''
        self.reject()
if __name__ == "__main__":
    import sys
    utils.set_language('ta')
    def except_hook(cls, exception, traceback):
        print('exception called')
        sys.__excepthook__(cls, exception, traceback)
    sys.excepthook = except_hook
    App = QApplication(sys.argv)
    chart = DhasaBhukthiOptionDialog(2,[True,False,2,13,0,0,0,-1])
    chart.show()
    sys.exit(App.exec())




from jhora.panchanga import drik
from jhora import const, utils

# Function to calculate the planet position using drik
def calculate_planet_position(jd, place, planet):
    if planet == 'L':
        sla = drik.ascendant(jd, place)
        return sla[0] * 30 + sla[1]
    else:
        return drik.sidereal_longitude(jd, planet)

# Lagrange Inverse Interpolation
def lagrange_inverse_interpolation(x, y, x_val):
    n = len(x)
    y_val = 0
    for i in range(n):
        term = y[i]
        for j in range(n):
            if i != j:
                term *= (x_val - x[j]) / (x[i] - x[j])
        y_val += term
    return y_val

# Function to find the conjunction time
def find_conjunction(planet1, planet2, jd, place, max_days=365*20, tolerance=1.0, tuning_factor=0.9):
    # Calculate initial positions
    lon1 = calculate_planet_position(jd, place, planet1)
    lon2 = calculate_planet_position(jd, place, planet2)
    print('initial longs', lon1, lon2)
    # Calculate initial longitude difference
    initial_diff = abs(lon1 - lon2)
    
    # Get planet velocities
    velocity1 = planet_speeds[planet1]
    velocity2 = planet_speeds[planet2]
    # Estimate time to close the gap
    differential_velocity = abs(velocity1 - velocity2)
    if differential_velocity == 0:
        return None  # Conjunction cannot be found if velocities are equal
    
    estimated_time = initial_diff / differential_velocity*tuning_factor
    jd += estimated_time  # Move time forward by the estimated time
    print(velocity1, velocity2, 'estimated starting date/time', utils.jd_to_gregorian(jd))
    
    # Start precise iterations from the new estimate
    time_step = 1.0 / 24.0 / 60.0 if planet1 == 'L' or planet2 == 'L' else 0.01
    for _ in range(int(max_days / (time_step ))):
        lon1 = calculate_planet_position(jd, place, planet1)
        lon2 = calculate_planet_position(jd, place, planet2)
        lon_diff = abs(lon1 - lon2)
        if lon_diff < tolerance:
            print(lon1, lon2, utils.jd_to_gregorian(jd))
            jd_list = [jd+t*time_step for t in range(-10,10)]
            long_diff_list = []
            for jdt in jd_list:
                p1_long = calculate_planet_position(jdt, place, planet1)
                p2_long = calculate_planet_position(jdt, place, planet2)
                #print(jd,jdt,planet1,p1_long,planet2,p2_long)
                long_diff_list.append((p1_long-p2_long))
            #print(jd_list)
            #print(long_diff_list)
            refined_jd = utils.inverse_lagrange(jd_list, long_diff_list, 0.0)
            refined_lon1 = calculate_planet_position(refined_jd, place, planet1)
            refined_lon2 = calculate_planet_position(refined_jd, place, planet2)
            print(refined_jd,utils.jd_to_gregorian(refined_jd),refined_lon1,refined_lon2)
            return refined_jd,refined_lon1

        jd += time_step

    return None  # Conjunction not found within max days

planet_speeds = {
    'L': 360, const._SUN: 1.0, const._MOON: 13.0, const._MARS: 0.4, 
    const._MERCURY: 1.4, const._JUPITER: 0.2, const._VENUS: 1.2, 
    const._SATURN: 0.008, const._RAHU: -0.05, const._KETU: -0.05
}
utils.set_ephemeris_data_path()
# Example usage
planet1 = const._JUPITER
planet2 = const._SUN
dob = drik.Date(1996, 12, 7)
tob = (10, 34, 0)
jd = utils.julian_day_number(dob, tob)
place = drik.Place('Chennai, India', 13.0878, 80.2785, 5.5)

conjunction_results = find_conjunction(planet1, planet2, jd, place)
if conjunction_results:
    conjunction_jd,conjunction_longitude = conjunction_results
    #conjunction_jd = conjunction_results
    cy,cm,cd,cfh = utils.jd_to_gregorian(conjunction_jd)
    print(planet1, planet2, "Conjunction will occur at:", (cy,cm,cd),utils.to_dms(cfh),utils.to_dms(conjunction_longitude,is_lat_long='plong'))
else:
    print(planet1, planet2, "Conjunction not found within the given time frame.")



from jhora.panchanga import drik
from jhora import utils

def find_conjunction(dob, tob, place):
    # Initialize Julian Day number
    jd = utils.julian_day_number(dob, tob)
    # Define the number of planets (0 to 8)
    num_planets = 9
    # Define a large step size for initial iterations
    step_size = 0.1  # Step size in Julian days, can be adjusted
    
    while True:
        # Get the full longitudes of all planets
        planet_longitudes = []
        for planet in range(num_planets):
            planet_sign, planet_long = drik.sidereal_longitude(jd, planet)
            full_long = planet_sign * 30 + planet_long
            planet_longitudes.append(full_long)
        
        # Check for conjunctions
        for i in range(num_planets):
            for j in range(i + 1, num_planets):
                if abs(planet_longitudes[i] - planet_longitudes[j]) < 0.01:  # Allow small tolerance for precision
                    return jd, i, j
        
        # Update Julian Day number by step size
        jd += step_size

# Example usage
dob = drik.Date(1996, 12, 7)
tob = (10, 34, 0)
place = drik.Place('Chennai, India', 13.0878, 80.2785, 5.5)
jd_conjunction, planet1, planet2 = find_conjunction(dob, tob, place)
print(f'Conjunction found at JD {jd_conjunction} between planet {planet1} and planet {planet2}')



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
        Calculates Tithi Ashtottari (=108) Dasha-bhukthi-antara-sukshma-prana
        Ref: https://www.indiadivine.org/content/topic/1488164-vedavyasa-tithi-ashtottari-dasa-tutorial/

"""

import swisseph as swe
from collections import OrderedDict as Dict
from jhora import const,utils
from jhora.panchanga import drik
from jhora.horoscope.chart import house
year_duration = const.sidereal_year  # some say 360 days, others 365.25 or 365.2563 etc
human_life_span_for_ashtottari_dhasa = 108
""" 
    {ashtottari adhipati:[(tithis),dasa_length]} 
"""
ashtottari_adhipathi_list = [0,1,2,3,6,4,7,5]
ashtottari_adhipathi_dict = {0:[(1,9,16,24),6],1:[(2,10,17,25),15],2:[(3,11,18,26),8],3:[(4,12,19,27),17],
                             6:[(7,15,22),10],4:[(5,13,20,28),19],7:[(8,23,30),12],5:[(6,14,21,29),21]}
def ashtottari_adhipathi(tithi_index):
    for key,(tithi_list,durn) in ashtottari_adhipathi_dict.items():
        if tithi_index in tithi_list:
            return key,durn 
def ashtottari_dasha_start_date(jd,place,tithi_index=1):
    _,_,_,birth_time_hrs = utils.jd_to_gregorian(jd)
    tit = drik.tithi(jd, place,tithi_index=tithi_index)
    t_frac = utils.get_fraction(tit[1], tit[2], birth_time_hrs)
    lord,res = ashtottari_adhipathi(tit[0])          # ruler of current nakshatra
    period_elapsed = (1-t_frac)*res*year_duration
    start_jd = jd - period_elapsed      # so many days before current day
    return [lord, start_jd]
def ashtottari_next_adhipati(lord,dirn=1):
    """Returns next lord after `lord` in the adhipati_list"""
    current = ashtottari_adhipathi_list.index(lord)
    next_index = (current + dirn) % len(ashtottari_adhipathi_list)
    return list(ashtottari_adhipathi_dict.keys())[next_index]
def ashtottari_mahadasa(jd,place,tithi_index):
    """
        returns a dictionary of all mahadashas and their start dates
        @return {mahadhasa_lord_index, (starting_year,starting_month,starting_day,starting_time_in_hours)}
    """
    lord, start_date = ashtottari_dasha_start_date(jd,place,tithi_index)
    retval = Dict()
    for _ in range(len(ashtottari_adhipathi_list)):
        retval[lord] = start_date
        lord_duration = ashtottari_adhipathi_dict[lord][1]
        start_date += lord_duration * year_duration
        lord = ashtottari_next_adhipati(lord)
    return retval
def ashtottari_bhukthi(dhasa_lord, start_date,antardhasa_option=3):
    """
        Compute all bhukthis of given nakshatra-lord of Mahadasa and its start date
    """
    lord = dhasa_lord
    if antardhasa_option in [3,4]:
        lord = ashtottari_next_adhipati(lord, dirn=1) 
    elif antardhasa_option in [5,6]:
        lord = ashtottari_next_adhipati(lord, dirn=-1) 
    dirn = 1 if antardhasa_option in [1,3,5] else -1
    dhasa_lord_duration = ashtottari_adhipathi_dict[dhasa_lord][1]
    retval = Dict()
    #lord = ashtottari_next_adhipati(dhasa_lord,dirn) # For Ashtottari first bhukkti starts from dhasa's next lord
    for _ in range(len(ashtottari_adhipathi_list)):
        retval[lord] = start_date
        lord_duration = ashtottari_adhipathi_dict[lord][1]
        factor = lord_duration * dhasa_lord_duration / human_life_span_for_ashtottari_dhasa
        start_date += factor * year_duration
        lord = ashtottari_next_adhipati(lord,dirn)
    return retval
def ashtottari_anthara(dhasa_lord, bhukthi_lord,bhukthi_lord_start_date):
    """
        Compute all bhukthis of given nakshatra-lord of Mahadasa, its bhukthi lord and bhukthi_lord's start date
    """
    dhasa_lord_duration = ashtottari_adhipathi_dict[dhasa_lord][1]
    retval = Dict()
    lord = ashtottari_next_adhipati(bhukthi_lord) # For Ashtottari first bhukkti starts from dhasa's next lord
    for _ in range(len(ashtottari_adhipathi_list)):
        retval[lord] = bhukthi_lord_start_date
        lord_duration = ashtottari_adhipathi_dict[lord][1]
        factor = lord_duration * dhasa_lord_duration / human_life_span_for_ashtottari_dhasa
        bhukthi_lord_start_date += factor * year_duration
        lord = ashtottari_next_adhipati(lord)
    return retval
def get_ashtottari_dhasa_bhukthi(jd, place,use_tribhagi_variation=False,include_antardhasa=True,
                                 tithi_index=1,antardhasa_option=3): #antardhasa starts from next lord
    """
        provides Tithi Ashtottari dhasa bhukthi for a given date in julian day (includes birth time)
        This is Ashtottari Dhasa based on tithi instead of nakshathra
        @param jd: Julian day for birthdate and birth time
        @param place: Place as tuple (place name, latitude, longitude, timezone) 
        @param use_tribhagi_variation: False (default), True means dhasa bhukthi duration in three phases 
        @param include_antardhasa True/False. Default=True 
        @param tithi_index: 1=>Janma Tithi 2=>Dhana 3=>Bhratri, 4=>Matri 5=Putra 6=>Satru 7=>Kalatra 8=>Mrutyu 
                        9=>Bhagya 10=>Karma 11=>Laabha 12=>Vyaya 
        @param antardhasa_option:
            1 => dhasa lord - forward
            2 => dhasa lord - backward
            3 => next dhasa lord - forward (Default)
            4 => next dhasa lord - backward
            5 => prev dhasa lord - forward
            6 => prev dhasa lord - backward
        @return: a list of [dhasa_lord,bhukthi_lord,bhukthi_start]
          Example: [ [7, 5, '1915-02-09'], [7, 0, '1917-06-10'], [7, 1, '1918-02-08'],...]
    """
    global human_life_span_for_ashtottari_dhasa
    _dhasa_cycles = 1
    _tribhagi_factor = 1
    if use_tribhagi_variation:
        _tribhagi_factor = 1./3.
        _dhasa_cycles = int(_dhasa_cycles/_tribhagi_factor)
        human_life_span_for_ashtottari_dhasa *= _tribhagi_factor
        for k,(v1,v2) in ashtottari_adhipathi_dict.items():
            ashtottari_adhipathi_dict[k] = [v1,round(v2*_tribhagi_factor,2)]
    dashas = ashtottari_mahadasa(jd,place,tithi_index)
    dhasa_bhukthi=[]
    for i in dashas:
        dhasa_lord = i
        if include_antardhasa:
            bhukthis = ashtottari_bhukthi(i, dashas[i],antardhasa_option)
            for j in bhukthis:
                bhukthi_lord = j
                jd1 = bhukthis[j]
                y, m, d, h = utils.jd_to_gregorian(jd1)
                date_str = '%04d-%02d-%02d' %(y,m,d)+' '+utils.to_dms(h,as_string=True)
                dhasa_bhukthi.append([dhasa_lord,bhukthi_lord,date_str])
        else:
            jd1 = dashas[i]
            y, m, d, h = utils.jd_to_gregorian(jd1)
            date_str = '%04d-%02d-%02d' %(y,m,d)+' '+utils.to_dms(h,as_string=True)
            dhasa_bhukthi.append([dhasa_lord,date_str])
            
    return dhasa_bhukthi
'------ main -----------'
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests._STOP_IF_ANY_TEST_FAILED = False
    pvr_tests.tithi_ashtottari_tests()